
BWI = {['bank'] = {['num'] = 0, ['rows'] = 0},
             ['reagent'] = {['num'] = 0, ['rows'] = 0},
             ['void'] = {},
             ['guild'] = {}}

--[[//////////////////////////////////////////////////

    MAIN INTERFACE FUNCTIONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]

    BWI.Main = {}
    function BW_CreateFrames()

        ----- Button bar with search
            BWI.Main.Frame.Create()

        ----- bank frame
            BWI.Main.Bank.Create()

        ------ reagent frame
            BWI.Main.Reagent.Create()

        ------ void frame
            BWI.Main.Void.Create()

        ------ guild frame
            BWI.Main.Guild.Create()

        ------ search frame
            BWI.Main.Search.Create()
    end

    function BW_ValidateBankBags(slots)

        local i, button, bankrow
        local numslots = (BW_ROW * BWI.bank.rows) + BWI.bank.num
        local lastbutton = BW.mainframe.bank.buttons[numslots]
        local bankrow = BW.mainframe.bank.buttons[BW_ROW * BWI.bank.rows + 1]

        if not slots or slots == numslots then return-- Already correct amount of slots
        elseif slots < numslots then -- Hide extra slots
            local difference = numslots - slots
            for i = numslots, (numslots - difference + 1), -1 do

                BW.mainframe.bank.buttons[i]:Hide()

                if BWI.bank.num == 1 then
                    BWI.bank.num = BW_ROW
                    BWI.bank.rows = BWI.bank.rows - 1
                else
                    BWI.bank.num = BWI.bank.num - 1
                end
            end
        else -- Add or Show missing slots
            for i = numslots + 1, slots do

                if BW.mainframe.bank.buttons[i] then
                    BW.mainframe.bank.buttons[i]:Show()
                    if BWI.bank.num == BW_ROW then
                        BWI.bank.rows = BWI.bank.rows + 1
                        BWI.bank.num = 1
                    else
                        BWI.bank.num = BWI.bank.num + 1
                    end
                    lastbutton = BW.mainframe.bank.buttons[i]
                else
                    button = BWI.Button.Create(i, BW.mainframe.bank.frame)
                    button:SetScript("OnEnter", BWI.Button.Enter)
                    button:SetScript("OnLeave", function(self) GameTooltip:Hide() end)

                    if BWI.bank.num == BW_ROW then -- Last column of the row
                        button:SetPoint("TOP", bankrow, "BOTTOM", 0, -BW_FRAMEWHITESPACE)
                        bankrow = button
                        BWI.bank.rows = BWI.bank.rows + 1
                        BWI.bank.num = 1
                    else
                        button:SetPoint("TOPLEFT", lastbutton, "TOPRIGHT", BW_FRAMEWHITESPACE, 0)
                        BWI.bank.num = BWI.bank.num + 1
                    end
                    BW.mainframe.bank.buttons[i] = button
                    lastbutton = button
                end
            end
        end
        BW.mainframe.bank:SetHeight(10 + (BW_BUTTON_HEIGHT * (BWI.bank.rows + 2)) + (BW_FRAMEWHITESPACE * BWI.bank.rows))
        BW.mainframe.bank:SetWidth(20 + (BW_BUTTON_HEIGHT * BW_ROW) + (BW_FRAMEWHITESPACE * (BW_ROW - 1)))
    end

    BWI.Main.Frame = {}
    function BWI.Main.Frame.Create()

        BW.mainframe = CreateFrame("Frame", "BWmainframe", UIParent)
        BW.mainframe:SetPoint("CENTER")
        BW.mainframe:SetHeight(25)
        BW.mainframe:SetWidth(15 + (32 * 5) + 192) -- options + searchbar
        BW.mainframe:SetFrameStrata("DIALOG")
        BW.mainframe.layout = 1

        BWI.Backdrop(BW.mainframe)
        BW.mainframe:SetMovable(true)
        BW.mainframe:SetUserPlaced(true)

        BW.mainframe:SetScript("OnMouseDown", BWI.Main.Frame.MouseDown)
        BW.mainframe:SetScript("OnMouseUp", BWI.Main.Frame.MouseUp)

        -- Create bank button
        BWI.Main.Frame.Bank()

        -- Create reagent button
        BWI.Main.Frame.Reagent()

        -- Create void button
        BWI.Main.Frame.Void()

        -- Create guild button
        BWI.Main.Frame.Guild()

        -- Create option button
        BWI.Main.Frame.Options()

        -- Search bar
        BWI.Main.Frame.Search()

        -- Create count button
        BWI.Main.Frame.Count()
    end

    function BWI.Main.Frame.Button(label, parent)
        local button = CreateFrame("Button", nil, parent or BW.mainframe)

        button:SetWidth(30)
        button:SetHeight(BW.mainframe:GetHeight())
        button:SetNormalFontObject("GameFontNormal")
        button:SetText(label)
        button:SetBackdrop({bgFile = "Interface\\Buttons\\WHITE8x8",  edgeFile = "Interface\\Buttons\\WHITE8x8", edgeSize = 1})
        button:SetBackdropBorderColor(0, 0, 0, 0)

        local highlight = button:CreateTexture(nil, "HIGHLIGHT")
        highlight:SetTexture("Interface\\Buttons\\WHITE8x8")
        highlight:SetHeight(2)
        highlight:SetVertexColor(1, 1, 1, 1)
        button:SetHighlightTexture(highlight)
        BWI.Main.Frame.ButtonHighlight(button, "BOTTOM")

        return button
    end

    function BWI.Main.Frame.ButtonHighlight(button, orientation)
        local highlight = button:GetHighlightTexture()

        highlight:ClearAllPoints()
        highlight:SetPoint(orientation.."LEFT", button, orientation.."LEFT")
        highlight:SetPoint(orientation.."RIGHT", button, orientation.."RIGHT")
    end

    function BWI.Main.Frame.ButtonOrder(orientation)

        BW.mainframe.bankbutton:ClearAllPoints()
        BW.mainframe.reagentbutton:ClearAllPoints()
        BW.mainframe.voidbutton:ClearAllPoints()
        BW.mainframe.guildbutton:ClearAllPoints()
        BW.mainframe.optionbutton:ClearAllPoints()
        BW.mainframe.searchbar:ClearAllPoints()
        BW.mainframe.countbutton:ClearAllPoints()

        if orientation == "LEFT" then
            BWI.Main.Frame.ButtonLeft("LEFT", "RIGHT")
        else
            BWI.Main.Frame.ButtonRight("RIGHT", "LEFT")
        end
    end

    function BWI.Main.Frame.ButtonLeft(startpoint, direction)

        BW.mainframe.bankbutton:SetPoint("TOP"..startpoint, BW.mainframe, "TOP"..startpoint)
        BW.mainframe.guildbutton:SetPoint("BOTTOM", BW.mainframe, "BOTTOM")
        BW.mainframe.reagentbutton:SetPoint("TOP"..startpoint, BW.mainframe.bankbutton, "TOP"..direction)
        BW.mainframe.voidbutton:SetPoint("TOP"..startpoint, BW.mainframe.reagentbutton, "TOP"..direction)
        BW.mainframe.guildbutton:SetPoint("TOP"..startpoint, BW.mainframe.voidbutton, "TOP"..direction)
        BW.mainframe.optionbutton:SetPoint("TOP"..startpoint, BW.mainframe.guildbutton, "TOP"..direction)
        BW.mainframe.searchbar:SetPoint("TOP"..startpoint, BW.mainframe.optionbutton, "TOP"..direction, 10, 0)
        BW.mainframe.countbutton:SetPoint("TOP"..startpoint, BW.mainframe.searchbar, "TOP"..direction)
    end

    function BWI.Main.Frame.ButtonRight(startpoint, direction)

        
        BW.mainframe.guildbutton:SetPoint("TOP"..startpoint, BW.mainframe, "TOP"..startpoint)
        BW.mainframe.guildbutton:SetPoint("BOTTOM", BW.mainframe, "BOTTOM")
        BW.mainframe.voidbutton:SetPoint("TOP"..startpoint, BW.mainframe.guildbutton, "TOP"..direction)
        BW.mainframe.reagentbutton:SetPoint("TOP"..startpoint, BW.mainframe.voidbutton, "TOP"..direction)
        BW.mainframe.bankbutton:SetPoint("TOP"..startpoint, BW.mainframe.reagentbutton, "TOP"..direction)
        BW.mainframe.optionbutton:SetPoint("TOP"..startpoint, BW.mainframe.bankbutton, "TOP"..direction)
        BW.mainframe.searchbar:SetPoint("TOP"..startpoint, BW.mainframe.optionbutton, "TOP"..direction, 10, 0)
        BW.mainframe.countbutton:SetPoint("TOP"..startpoint, BW.mainframe.searchbar, "TOP"..direction)
    end

    function BWI.Main.Frame:MouseUp(...)
        self:StopMovingOrSizing()
    end

    function BWI.Main.Frame:MouseDown(...)
        self:StartMoving()
    end

    function BWI.Main.Frame.Bank()

        BW.mainframe.bankbutton = BWI.Main.Frame.Button("|cffffffffB|r")
        BW.mainframe.bankbutton:SetPoint("TOPLEFT", BW.mainframe, "TOPLEFT")
        BW.mainframe.bankbutton:SetPoint("BOTTOM", BW.mainframe, "BOTTOM")
        BW.mainframe.bankbutton:SetBackdropColor(0.32, 0.15, 0.64, 1)
        BW.mainframe.bankbutton:SetScript("OnClick", function(self)
            BW.mainframe.lastbank = nil
            BW.mainframe.reagent:Hide()
            BW.mainframe.void:Hide()
            BW.mainframe.guild:Hide()
            BW.mainframe.search:Hide()

            if BW.mainframe.bank:IsVisible() then
                BW.mainframe.bank:Hide()
            else
                BW.mainframe.lastbank = "bank"
                BW.mainframe.bank:Show()
            end
        end)
    end

    function BWI.Main.Frame.Reagent()

        BW.mainframe.reagentbutton = BWI.Main.Frame.Button("|cffffffffR|r")
        BW.mainframe.reagentbutton:SetPoint("TOPLEFT", BW.mainframe.bankbutton, "TOPRIGHT")
        BW.mainframe.reagentbutton:SetBackdropColor(0.34, 0.24, 0.49, 1)
        BW.mainframe.reagentbutton:SetScript("OnClick", function(self)
            BW.mainframe.lastbank = nil
            BW.mainframe.bank:Hide()
            BW.mainframe.void:Hide()
            BW.mainframe.guild:Hide()
            BW.mainframe.search:Hide()

            if BW.mainframe.reagent:IsVisible() then
                BW.mainframe.reagent:Hide()
            else
                BW.mainframe.lastbank = "reagent"
                BW.mainframe.reagent:Show()
            end
        end)
    end

    function BWI.Main.Frame.Void()

        BW.mainframe.voidbutton = BWI.Main.Frame.Button("|cffffffffV|r")
        BW.mainframe.voidbutton:SetPoint("TOPLEFT", BW.mainframe.reagentbutton, "TOPRIGHT")
        BW.mainframe.voidbutton:SetBackdropColor(0.24, 0.24, 0.49, 1)
        BW.mainframe.voidbutton:SetScript("OnClick", function(self)
            BW.mainframe.lastbank = nil
            BW.mainframe.bank:Hide()
            BW.mainframe.reagent:Hide()
            BW.mainframe.guild:Hide()
            BW.mainframe.search:Hide()

            if BW.mainframe.void:IsVisible() then
                BW.mainframe.void:Hide()
            else
                BW.mainframe.lastbank = "void"
                BW.mainframe.void:Show()
            end
        end)
    end

    function BWI.Main.Frame.Guild()

        BW.mainframe.guildbutton = BWI.Main.Frame.Button("|cffffffffG|r")
        BW.mainframe.guildbutton:SetPoint("TOPLEFT", BW.mainframe.voidbutton, "TOPRIGHT")
        BW.mainframe.guildbutton:SetBackdropColor(0.20, 0.17, 0.56, 1)
        BW.mainframe.guildbutton:SetScript("OnClick", function(self)
            BW.mainframe.lastbank = nil
            BW.mainframe.bank:Hide()
            BW.mainframe.reagent:Hide()
            BW.mainframe.void:Hide()
            BW.mainframe.search:Hide()

            if BW.mainframe.guild:IsVisible() then
                BW.mainframe.guild:Hide()
            else
                BW.mainframe.lastbank = "guild"
                BW.mainframe.guild:Show()
            end
        end)
    end

    function BWI.Main.Frame.Options()

        BW.mainframe.optionbutton = BWI.Main.Frame.Button("|cff7E7E7Eo|r")
        BW.mainframe.optionbutton:SetHighlightTexture("")
        BW.mainframe.optionbutton:SetPoint("TOPLEFT", BW.mainframe.guildbutton, "TOPRIGHT")
        BW.mainframe.optionbutton:SetBackdropColor(0.05, 0.05, 0.05, 1)
        BW.mainframe.optionbutton:RegisterForClicks("LeftButtonUp", "RightButtonUp")
        BW.mainframe.optionbutton:RegisterForDrag("LeftButton")
        BW.mainframe.optionbutton:SetScript("OnShow", function(self) BW_SetFrameLocation() end)
        BW.mainframe.optionbutton:SetScript("OnEnter", function(self) self:SetText("|cffffffffo|r") end)
        BW.mainframe.optionbutton:SetScript("OnLeave", function(self) self:SetText("|cff7E7E7Eo|r") end)
        BW.mainframe.optionbutton:SetScript("OnDragStart", function(self) self:SetText("|cff0FBCE2o|r"); BW.mainframe:StartMoving() end)
        BW.mainframe.optionbutton:SetScript("OnDragStop", function(self) self:SetText("|cff7E7E7Eo|r"); BW.mainframe:StopMovingOrSizing(); BW_SetFrameLocation(); end)
        BW.mainframe.optionbutton:SetScript("OnClick", function(self, button)
            if button == "RightButton" then
                BW.mainframe.togglebutton:Show()
            else 
                BWdropDownMenuButton:Click()
            end 
        end)

        BWI.OptionsDropDown.Create()
        BWI.Main.Frame.Toggle()
    end

    function BWI.Main.Frame.Toggle()

        BW.mainframe.togglebutton = BWI.Main.Frame.Button("|cffE62C2Co|r", UIParent)
        BW.mainframe.togglebutton:SetHighlightTexture("")
        BW.mainframe.togglebutton:SetAllPoints(BW.mainframe.optionbutton)
        BW.mainframe.togglebutton:Hide()
        BW.mainframe.togglebutton:SetBackdropColor(0.07, 0.07, 0.07, 0.8)
        BW.mainframe.togglebutton:SetBackdropBorderColor(0.15, 0.15, 0.15, 1)
        BW.mainframe.togglebutton:SetScript("OnEnter", function(self) self:SetText("|cff03C926o|r"); self:SetBackdropBorderColor(0.01, 0.79, 0.15, 1) end)
        BW.mainframe.togglebutton:SetScript("OnLeave", function(self) self:SetText("|cffE62C2Co|r"); self:SetBackdropBorderColor(0.15, 0.15, 0.15, 1) end)
        BW.mainframe.togglebutton:SetScript("OnShow", function(self) BW.mainframe:Hide() end)
        BW.mainframe.togglebutton:SetScript("OnHide", function(self) BW.mainframe:Show() end)
        BW.mainframe.togglebutton:SetScript("OnClick", function(self) BW.mainframe.optionbutton.hide = nil; self:Hide() end)
    end

    BWI.OptionsDropDown = {}
    function BWI.OptionsDropDown.Create()

        BWdropDownMenu = CreateFrame("Frame", "BWdropDownMenu", BW.mainframe, "UIDropDownMenuTemplate")
        BWdropDownMenu:Hide()
        UIDropDownMenu_SetWidth(BWdropDownMenu, 130, 4)
        BWdropDownMenu:SetPoint("TOPLEFT", BW.mainframe.optionbutton, "BOTTOMLEFT", -15, 24)
        UIDropDownMenu_Initialize(BWdropDownMenu, BWI.OptionsDropDown.Initialize) --The virtual
    end

    function BWI.OptionsDropDown:Initialize(level)
        --drop down menues can have sub menues. The value of level determines the drop down sub menu tier
        local level = level or 1
        local x, k, v

        if level == 1 then
            local info = UIDropDownMenu_CreateInfo()
            if BWsavedtable then
                for k, v in pairs(BWsavedtable) do
                    if not BW_OPT_HIDDEN[k] then BWI.OptionsDropDown.Menu(info, k, k, level) end
                end
            end
        elseif level == 2 then
            local info = UIDropDownMenu_CreateInfo()
            if BWsavedtable then
                for k, v in pairs(BWsavedtable[UIDROPDOWNMENU_MENU_VALUE]) do
                    checked = tostring(BW_ACTIVE_TABLE.player.."."..string.sub(BW_ACTIVE_TABLE.faction, 1, 1)) == k and true or false
                    player, faction = k:match"([^.]*).(.*)"
                    BWI.OptionsDropDown.Button(self, info, player.." ("..string.sub(faction, 1, 1)..")", UIDROPDOWNMENU_MENU_VALUE.."."..k, checked, level)
                end
            end
        end
    end

    function BWI.OptionsDropDown.Menu(info, label, value, level)

        info.text = label
        info.value = value
        info.hasArrow = true
        info.checked = false

        UIDropDownMenu_AddButton(info, level)
    end

    function BWI.OptionsDropDown.Button(self, info, label, value, checked, level)

        info.text = label
        info.hasArrow = false
        info.value = value
        info.func = BWdropDownMenuItem_OnClick
        info.owner = self:GetParent()
        info.checked = checked

        UIDropDownMenu_AddButton(info, level)
    end

    function BWI.Main.Frame.Search()

        BW.mainframe.searchbar = CreateFrame("EditBox", "BW_Searchbar", BW.mainframe, "BagSearchBoxTemplate")
        BW.mainframe.searchbar:SetHeight(25)
        BW.mainframe.searchbar:SetWidth(192)
        BW.mainframe.searchbar:SetAutoFocus(false)
        BW.mainframe.searchbar:SetPoint("TOPLEFT", BW.mainframe.optionbutton, "TOPRIGHT", 10, 0)
        BW.mainframe.searchbar:SetHyperlinksEnabled(true) 
        BW.mainframe.searchbar:HookScript("OnEscapePressed", function(self) self:ClearFocus() end)
        BW.mainframe.searchbar:HookScript("OnTextChanged", function(self)
            text = self:GetText()
            if text == "" then text = false end

            if text ~= SEARCH then

                BW.mainframe:SetBackdropColor(0.05, 0.05, 0.05, 1)
                BW.mainframe.countbutton.count = 0
                BW.mainframe.search.scroll.ScrollBar:SetValue(0)

                BW_Search(text)

                if BW.mainframe.countbutton.count > 0 then
                    BW.mainframe.countbutton.labelcount:SetText("|cffffffff"..BW.mainframe.countbutton.count.."|r")
                    BW.mainframe.countbutton:Show()
                else
                    BW.mainframe.countbutton:Hide()
                end

                if not text then
                    BW.mainframe:SetBackdropColor(0, 0, 0, 0.5)
                end
            else
                BW.mainframe:SetBackdropColor(0, 0, 0, 0.5)
                BW_Search(false)
            end
        end)
        BW_RemoveEdge(BW.mainframe.searchbar)
        tinsert(ITEM_SEARCHBAR_LIST, "BW_Searchbar")
    end

    function BWI.Main.Frame.Count()

        BW.mainframe.countbutton = BWI.Main.Frame.Button("|cffffffff|r")
        BW.mainframe.countbutton:SetWidth(64)
        BW.mainframe.countbutton:SetPoint("TOPLEFT", BW.mainframe.searchbar, "TOPRIGHT")
        BW.mainframe.countbutton:SetBackdropColor(0.05, 0.05, 0.05, 1)
        BW.mainframe.countbutton:Hide()

        BW.mainframe.countbutton:SetScript("OnHide", function(self) BW.mainframe.search:Hide() end)
        BW.mainframe.countbutton:SetScript("OnClick", function(self)
            BW.mainframe.bank:Hide()
            BW.mainframe.reagent:Hide()
            BW.mainframe.void:Hide()
            BW.mainframe.guild:Hide()

            BW_SetFrameLocation()

            if BW.mainframe.search:IsVisible() then
                BW.mainframe.search:Hide()
                if BW.mainframe.lastbank then BW.mainframe[BW.mainframe.lastbank]:Show() end
            else
                BW.mainframe.search:Show()
            end
        end)

        BW.mainframe.countbutton.labelcount = BW.mainframe.countbutton:CreateFontString(nil, "OVERLAY", "GameFontNormal")
        BW.mainframe.countbutton.labelcount:SetFont(BW.mainframe.countbutton.labelcount:GetFont(), 14) -- Resize string
        BW.mainframe.countbutton.labelcount:SetJustifyH("CENTER")
        BW.mainframe.countbutton.labelcount:SetPoint("CENTER")
    end

    BWI.Main.Search = {}
    function BWI.Main.Search.Create()

        BW.mainframe.search = CreateFrame("Frame", "BWsearch", BW.mainframe)
        BW.mainframe.search:SetHeight(200)
        BW.mainframe.search:SetPoint("TOPLEFT", BW.mainframe.optionbutton, "BOTTOMLEFT")
        BW.mainframe.search:SetPoint("TOPRIGHT", BW.mainframe.countbutton, "BOTTOMRIGHT")

        BW.mainframe.search:Hide()
        BW.mainframe.search:EnableMouse(true)

        BW.mainframe.search.enddot = BW.mainframe.search:CreateTexture(nil, "OVERLAY")
        BW.mainframe.search.enddot:SetMask("Textures\\MinimapMask")
        BW.mainframe.search.enddot:SetHeight(5)
        BW.mainframe.search.enddot:SetWidth(5)
        BW.mainframe.search.enddot:SetTexture("Interface\\Buttons\\WHITE8x8")
        BW.mainframe.search.enddot:SetVertexColor(0.7, 0.7, 0.7, 1)
        BW.mainframe.search.enddot:SetPoint("TOP", BW.mainframe.search, "BOTTOM", 0, -5)

        BWI.Main.Search.Scroll()
    end

    function BWI.Main.Search.Scroll()

        BW.mainframe.search.list = CreateFrame("Frame", "FauxScrollFrameBW", BW.mainframe.search)
        BW.mainframe.search.list:SetPoint("TOPLEFT", BW.mainframe.search, "TOPLEFT", 0, -BW_FRAMEWHITESPACE)
        BW.mainframe.search.list:SetPoint("BOTTOMRIGHT", BW.mainframe.search, "BOTTOMRIGHT")
        BW.mainframe.search.list:SetWidth(BW.mainframe.search:GetWidth())

        BW.mainframe.search.scroll = BW_ScrollFrame(BW.mainframe.search.list, "BW_scrollframe",
                                                       BW_ScrollbarUpdate, BWI.Main.Search.Button)
    end

    function BWI.Main.Search.Button(index)

        local button = BWI.List.Button(index, BW.mainframe.search.list)

        button:SetScript("OnEnter", BWI.Button.Enter)
        button:SetScript("OnLeave", function(self) GameTooltip:Hide() end)
        button.leftstring = button:CreateFontString(nil, "OVERLAY", "GameFontNormal")
        button.leftstring:SetJustifyH("LEFT")
        button.leftstring:SetJustifyV("CENTER")
        button.leftstring:SetWordWrap(false)
        button.leftstring:SetPoint("LEFT", button.icon, "RIGHT", 10, 0)
        button.leftstring:SetPoint("RIGHT", button, "RIGHT", -BW.mainframe.countbutton:GetWidth(), 0)

        button:SetScript("OnClick", function(self) 
            local location = self.location.loc
            local page = tonumber(self.location.num)

            if location == "bank" then
                BW.mainframe.bankbutton:Click()
            elseif location == "reagent" then
                BW.mainframe.reagentbutton:Click()
            elseif location == "void" then
                BW.mainframe.voidbutton:Click()
                BW.mainframe.void[page].tab:Click()
            elseif location == "guild" then
                BW.mainframe.guildbutton:Click()
                BW.mainframe.guild[page].tab:Click()
            end
        end)

       return button
    end

    BWI.Main.Bank = {}
    function BWI.Main.Bank.Create()

        BW.mainframe.bank = CreateFrame("Frame", "BWbank", BW.mainframe)
        BW.mainframe.bank:SetPoint("TOPLEFT", BW.mainframe, "BOTTOMLEFT")
        BW.mainframe.bank:SetWidth(20 + (BW_BUTTON_HEIGHT * BW_ROW) + (BW_FRAMEWHITESPACE * (BW_ROW - 1)))
        BW.mainframe.bank:SetFrameStrata("HIGH")
        BW.mainframe.bank:Hide()

        BW.mainframe.bank:SetScript("OnShow", function(self) BW.mainframe.bankbutton:LockHighlight() end)
        BW.mainframe.bank:SetScript("OnHide", function(self) BW.mainframe.bankbutton:UnlockHighlight() end)

        BWI.Backdrop(BW.mainframe.bank)
    end

    function BWI.Main.Bank.Buttons()

        BW.mainframe.bank.frame = CreateFrame("Frame", nil, BW.mainframe.bank)
        BW.mainframe.bank.frame:SetPoint("BOTTOM")
        BW.mainframe.bank.frame:SetPoint("TOPLEFT", 10, -20)
        BW.mainframe.bank.frame:SetFrameStrata("HIGH")

        BW.mainframe.bank.buttons = {}
        local i, button, bankrow, lastbutton

        for i = 1, NUM_BANKGENERIC_SLOTS do

            button = BWI.Button.Create(i, BW.mainframe.bank.frame)
            button:SetScript("OnEnter", BWI.Button.Enter)
            button:SetScript("OnLeave", function(self) GameTooltip:Hide() end)
            
            if i == 1 then -- First slot
                button:SetPoint("TOPLEFT")
                BWI.bank.num = 1
                bankrow = button

            elseif BWI.bank.num == BW_ROW then -- Last column of the row
                button:SetPoint("TOP", bankrow, "BOTTOM", 0, -BW_FRAMEWHITESPACE)
                bankrow = button
                BWI.bank.rows = BWI.bank.rows + 1
                BWI.bank.num = 1
            else
                button:SetPoint("TOPLEFT", lastbutton, "TOPRIGHT", BW_FRAMEWHITESPACE, 0)
                BWI.bank.num = BWI.bank.num + 1
            end
            BW.mainframe.bank.buttons[i] = button
            lastbutton = button
        end
        BW.mainframe.bank:SetHeight(10 + (BW_BUTTON_HEIGHT * (BWI.bank.rows + 2)) + (BW_FRAMEWHITESPACE * BWI.bank.rows))
        BW.mainframe.bank:SetWidth(20 + (BW_BUTTON_HEIGHT * BW_ROW) + (BW_FRAMEWHITESPACE * (BW_ROW - 1)))
    end

    BWI.Main.Reagent = {}
    function BWI.Main.Reagent.Create()

        BW.mainframe.reagent = CreateFrame("Frame", "BWreagent", BW.mainframe)
        BW.mainframe.reagent:SetPoint("TOPLEFT", BW.mainframe, "BOTTOMLEFT")
        BW.mainframe.reagent:SetFrameStrata("HIGH")
        BW.mainframe.reagent:Hide()

        BW.mainframe.reagent:SetScript("OnShow", function(self) BW.mainframe.reagentbutton:LockHighlight() end)
        BW.mainframe.reagent:SetScript("OnHide", function(self) BW.mainframe.reagentbutton:UnlockHighlight() end)

        BWI.Backdrop(BW.mainframe.reagent)
    end

    function BWI.Main.Reagent.Buttons()

        BW.mainframe.reagent.buttons = {}
        local i, x, rows, num, button, bankrow, lastbutton

        for x = 1, BW_REAGENT_PAGES do
            BW.mainframe.reagent[x] = {}
            BW.mainframe.reagent[x].buttons = {}
            BW.mainframe.reagent[x].frame = CreateFrame("Frame", nil, BW.mainframe.reagent)
            BW.mainframe.reagent[x].frame:SetPoint("BOTTOM")
            BW.mainframe.reagent[x].frame:SetPoint("TOPLEFT", 10, -20)
            BW.mainframe.reagent[x].frame:SetFrameStrata("HIGH")
            if x ~= 1 then BW.mainframe.reagent[x].frame:Hide() end

            rows = 0
            num = 0

            for i = 1, BW_NUM_REAGENT_SLOTS do

                button = BWI.Button.Create(i, BW.mainframe.reagent[x].frame)
                button:SetScript("OnEnter", BWI.Button.Enter)
                button:SetScript("OnLeave", function(self) GameTooltip:Hide() end)
                
                if i == 1 then -- First slot
                    button:SetPoint("TOPLEFT")
                    rows = rows + 1
                    num = 1
                    bankrow = button

                elseif num == BW_ROW then -- Last column of the row
                    button:SetPoint("TOP", bankrow, "BOTTOM", 0, -BW_FRAMEWHITESPACE)
                    rows = rows + 1
                    num = 1
                    bankrow = button
                else
                    button:SetPoint("TOPLEFT", lastbutton, "TOPRIGHT", BW_FRAMEWHITESPACE, 0)
                    num = num + 1
                end
                BW.mainframe.reagent[x].buttons[i] = button
                lastbutton = button
            end
        end
        BW.mainframe.reagent:SetHeight(10 + (BW_BUTTON_HEIGHT * (rows + 1)) + (BW_FRAMEWHITESPACE * rows))
        BW.mainframe.reagent:SetWidth(20 + (BW_BUTTON_HEIGHT * BW_ROW) + (BW_FRAMEWHITESPACE * (BW_ROW - 1)))
    end

    BWI.Main.Void = {}
    function BWI.Main.Void.Create()

        BW.mainframe.void = CreateFrame("Frame", "BWvoid", BW.mainframe)
        BW.mainframe.void:SetPoint("TOPLEFT", BW.mainframe, "BOTTOMLEFT")
        BW.mainframe.void:SetWidth(20 + (BW_BUTTON_HEIGHT * BW_ROW) + (BW_FRAMEWHITESPACE * (BW_ROW - 1)))
        BW.mainframe.void:SetFrameStrata("HIGH")
        BW.mainframe.void.lasttab = 1
        BW.mainframe.void:Hide()

        BW.mainframe.void:SetScript("OnShow", function(self) BW.mainframe.voidbutton:LockHighlight() end)
        BW.mainframe.void:SetScript("OnHide", function(self) BW.mainframe.voidbutton:UnlockHighlight() end)

        BWI.Backdrop(BW.mainframe.void)
        local x, lasttab

        for x = 1, BW_VOID_PAGES do
            BW.mainframe.void[x] = {}
            BW.mainframe.void[x].tab = BWI.Tab.Create(BW.mainframe.void)
            BW.mainframe.void[x].tab.index = x
            BW.mainframe.void[x].tab:SetScript("OnClick", function(self)
                BW.mainframe.void[BW.mainframe.void.lasttab].frame:Hide()
                BW.mainframe.void[BW.mainframe.void.lasttab].tab:SetBackdropBorderColor(0, 0, 0, 1)
                self:SetBackdropBorderColor(1, 1, 1, 1)
                BW.mainframe.void[self.index].frame:Show()
                BW.mainframe.void.lasttab = self.index
            end)

            if x == 1 then
                BW.mainframe.void[x].tab:SetBackdropBorderColor(1, 1, 1, 1)
                BW.mainframe.void[x].tab:SetNormalTexture("Interface\\Icons\\INV_Enchant_EssenceCosmicGreater")
                BW.mainframe.void[x].tab:SetPoint("TOPLEFT", BW.mainframe.void, "TOPRIGHT", 0, -1)
            else
                BW.mainframe.void[x].tab:SetNormalTexture("Interface\\Icons\\INV_Enchant_EssenceArcaneLarge")
                BW.mainframe.void[x].tab:SetPoint("TOPLEFT", lasttab, "BOTTOMLEFT", 0, -4)
            end
            lasttab = BW.mainframe.void[x].tab
        end
    end

    function BWI.Main.Void.Buttons()

        local i, x, button, lastrow, rows, num

        for x = 1, BW_VOID_PAGES do
            BW.mainframe.void[x].buttons = {}
            BW.mainframe.void[x].frame = CreateFrame("Frame", nil, BW.mainframe.void)
            BW.mainframe.void[x].frame:SetPoint("BOTTOM")
            BW.mainframe.void[x].frame:SetPoint("TOPLEFT", 10, -20)
            BW.mainframe.void[x].frame:SetFrameStrata("HIGH")
            if x ~= 1 then BW.mainframe.void[x].frame:Hide() end

            rows = 0
            num = 0

            for i = 1, BW_NUM_VOID_SLOTS do

                button = BWI.Button.Create(i, BW.mainframe.void[x].frame)
                button:SetScript("OnEnter", BWI.Button.Enter)
                button:SetScript("OnLeave", function(self) GameTooltip:Hide() end)
                
                if i == 1 then -- First slot
                    button:SetPoint("TOPLEFT")
                    rows = rows + 1
                    num = 1
                    lastrow = button

                elseif num == BW_ROW then -- Last column of the row
                    button:SetPoint("TOP", lastrow, "BOTTOM", 0, -BW_FRAMEWHITESPACE)
                    rows = rows + 1
                    num = 1
                    lastrow = button
                else
                    button:SetPoint("TOPLEFT", lastbutton, "TOPRIGHT", BW_FRAMEWHITESPACE, 0)
                    num = num + 1
                end
                BW.mainframe.void[x].buttons[i] = button
                lastbutton = button
            end
        end
        BW.mainframe.void:SetHeight(10 + (BW_BUTTON_HEIGHT * (rows + 1)) + (BW_FRAMEWHITESPACE * rows))
        BW.mainframe.void:SetWidth(20 + (BW_BUTTON_HEIGHT * BW_ROW) + (BW_FRAMEWHITESPACE * (BW_ROW - 1)))
    end

    BWI.Main.Guild = {}
    function BWI.Main.Guild.Create()

        BW.mainframe.guild = CreateFrame("Frame", "BWguild", BW.mainframe)
        BW.mainframe.guild:SetPoint("TOPLEFT", BW.mainframe, "BOTTOMLEFT")
        BW.mainframe.guild:SetWidth(20 + (BW_BUTTON_HEIGHT * BW_ROW) + (BW_FRAMEWHITESPACE * (BW_ROW - 1)))
        BW.mainframe.guild:SetFrameStrata("HIGH")
        BW.mainframe.guild.lasttab = 1
        BW.mainframe.guild:Hide()

        BW.mainframe.guild:SetScript("OnShow", function(self) BW.mainframe.guildbutton:LockHighlight() end)
        BW.mainframe.guild:SetScript("OnHide", function(self) BW.mainframe.guildbutton:UnlockHighlight() end)

        BWI.Backdrop(BW.mainframe.guild)
        local x, lasttab

        for x = 1, MAX_GUILDBANK_TABS do
            BW.mainframe.guild[x] = {}
            BW.mainframe.guild[x].tab = BWI.Tab.Create(BW.mainframe.guild)
            BW.mainframe.guild[x].tab.index = x
            BW.mainframe.guild[x].tab:SetScript("OnClick", function(self)
                BW.mainframe.guild[BW.mainframe.guild.lasttab].frame:Hide()
                BW.mainframe.guild[BW.mainframe.guild.lasttab].tab:SetBackdropBorderColor(0, 0, 0, 1)
                self:SetBackdropBorderColor(1, 1, 1, 1)
                BW.mainframe.guild[self.index].frame:Show()
                BW.mainframe.guild.lasttab = self.index
            end)
            BW.mainframe.guild[x].tab:SetScript("OnEnter", function(self) 
                GameTooltip:SetOwner(self, "ANCHOR_RIGHT", -10, -10)
                if self.name then
                    GameTooltip:SetText(self.name)
                end
            end)
            BW.mainframe.guild[x].tab:SetScript("OnLeave", function(self) GameTooltip:Hide() end)

            if x == 1 then
                BW.mainframe.guild[x].tab:SetPoint("TOPLEFT", BW.mainframe.guild, "TOPRIGHT", 0, -1)
                BW.mainframe.guild[x].tab:SetBackdropBorderColor(1, 1, 1, 1)
            else
                BW.mainframe.guild[x].tab:SetPoint("TOPLEFT", lasttab, "BOTTOMLEFT", 0, -4)
            end
            lasttab = BW.mainframe.guild[x].tab
        end
    end

    function BWI.Main.Guild.Buttons()

        local i, x, button, lastrow, rows, num

        for x = 1, MAX_GUILDBANK_TABS do
            BW.mainframe.guild[x].buttons = {}
            BW.mainframe.guild[x].frame = CreateFrame("Frame", nil, BW.mainframe.guild)
            BW.mainframe.guild[x].frame:SetPoint("BOTTOM")
            BW.mainframe.guild[x].frame:SetPoint("TOPLEFT", 10, -20)
            BW.mainframe.guild[x].frame:SetFrameStrata("HIGH")
            if x ~= 1 then
                BW.mainframe.guild[x].frame:Hide()
            end

            rows = 0
            num = 0

            for i = 1, MAX_GUILDBANK_SLOTS_PER_TAB do

                button = BWI.Button.Create(i, BW.mainframe.guild[x].frame)
                button:SetScript("OnEnter", BWI.Button.Enter)
                button:SetScript("OnLeave", function(self) GameTooltip:Hide() end)
                
                if i == 1 then -- First slot
                    button:SetPoint("TOPLEFT")
                    rows = rows + 1
                    num = 1
                    lastrow = button

                elseif num == BW_ROW then -- Last column of the row
                    button:SetPoint("TOP", lastrow, "BOTTOM", 0, -BW_FRAMEWHITESPACE)
                    rows = rows + 1
                    num = 1
                    lastrow = button
                else
                    button:SetPoint("TOPLEFT", lastbutton, "TOPRIGHT", BW_FRAMEWHITESPACE, 0)
                    num = num + 1
                end
                BW.mainframe.guild[x].buttons[i] = button
                lastbutton = button
            end
        end
        BW.mainframe.guild:SetHeight(10 + (BW_BUTTON_HEIGHT * (rows + 1)) + (BW_FRAMEWHITESPACE * rows))
        BW.mainframe.guild:SetWidth(20 + (BW_BUTTON_HEIGHT * BW_ROW) + (BW_FRAMEWHITESPACE * (BW_ROW - 1)))
    end

    BWI.Button = {}
    function BWI.Button.Create(index, parent)

        local button = CreateFrame("Button", nil, parent)

        button.buttonnumber = index
        button:SetHeight(BW_BUTTON_HEIGHT)
        button:SetWidth(BW_BUTTON_HEIGHT)
        button:SetFrameStrata("HIGH")

        button:SetBackdrop({bgFile = "Interface\\Buttons\\WHITE8x8", edgeFile = "Interface\\Buttons\\WHITE8x8", edgeSize = 1})
        button:SetBackdropColor(0, 0, 0, 0.2)
        button:SetBackdropBorderColor(0, 0, 0, 1)

        local icon = button:CreateTexture(nil, "ARTWORK", -1)

        icon:SetHeight(BW_BUTTON_HEIGHT - 2)
        icon:SetWidth(BW_BUTTON_HEIGHT - 2)
        icon:SetTexture("Interface\\AddOns\\BanksWatch\\media\\gloss")
        icon:SetTexCoord(0.1, 0.9, 0.1, 0.9)
        icon:SetPoint("CENTER")
        button.icon = icon

        local search = button:CreateTexture(nil, "OVERLAY")
        search:SetAllPoints(icon)
        search:SetTexture("Interface\\Buttons\\WHITE8x8")
        search:SetVertexColor(0, 0, 0, 0.7)
        search:Hide()
        button.search = search

        local highlight = button:CreateTexture(nil, "HIGHLIGHT")

        highlight:SetAllPoints()
        highlight:SetTexture("Interface\\Buttons\\ButtonHilight-Square")
        highlight:SetVertexColor(0.949, 0.847, 0.745, 1)
        button:SetHighlightTexture(highlight)

        button.text = button:CreateFontString(nil, "ARTWORK", "GameFontNormal")
        button.text:SetTextColor(1, 1, 1)
        button.text:SetFont(button.text:GetFont(), 14, "OUTLINE")
        button.text:SetJustifyH("LEFT")
        button.text:SetJustifyV("CENTER")
        button.text:SetWordWrap(false)
        button.text:SetPoint("RIGHT")
        button.text:SetPoint("BOTTOM", 0, 3)

        button.ilvl = button:CreateFontString(nil, "ARTWORK", "GameFontNormal")
        button.ilvl:SetFont(button.ilvl:GetFont(), 12, "OUTLINE")
        button.ilvl:SetJustifyH("LEFT")
        button.ilvl:SetJustifyV("CENTER")
        button.ilvl:SetWordWrap(false)
        button.ilvl:SetPoint("LEFT", 1)
        button.ilvl:SetPoint("BOTTOM", 0, 2)

        button:SetScript("OnClick", function(self) 
            if IsShiftKeyDown() then
                if BW.mainframe.searchbar:HasFocus() then
                    BW.mainframe.searchbar:SetText(self.name)
                else
                    SetItemRef(self.link, self.link, "LeftButton")
                end
            end
        end)

        return button
    end

    BWI.List = {}
    function BWI.List.Button(index, parent)

        local button = CreateFrame("Button", nil, parent)

        button.buttonnumber = index
        button:SetHeight(32)
        button:SetWidth(parent:GetWidth())
        button:SetPoint("TOP")
        button:SetPoint("LEFT")
        button:SetPoint("RIGHT")
        button:SetBackdrop({bgFile = "Interface\\Buttons\\WHITE8x8", edgeFile = "Interface\\Buttons\\WHITE8x8", edgeSize = 2})
        button:SetBackdropColor(0, 0, 0, 0.7)
        button:SetBackdropBorderColor(0, 0, 0, 0.9)

        local highlight = button:CreateTexture(nil, "HIGHLIGHT")

        highlight:SetHeight(button:GetHeight() - 2)
        highlight:SetWidth(button:GetWidth() - 2)
        highlight:SetPoint("CENTER")
        highlight:SetTexture("Interface\\Buttons\\WHITE8x8")
        highlight:SetVertexColor(0.7, 0.7, 0.7, 0.05)
        button:SetHighlightTexture(highlight)

        local icon = button:CreateTexture(nil, "ARTWORK", -1)

        icon:SetHeight(28)
        icon:SetWidth(28)
        icon:SetTexture("Interface\\AddOns\\BanksWatch\\media\\gloss")
        icon:SetTexCoord(0.1, 0.9, 0.1, 0.9)
        icon:SetPoint("TOPLEFT", 2, -2)
        button.icon = icon

        button.text = button:CreateFontString(nil, "ARTWORK", "GameFontNormal")
        button.text:SetTextColor(1, 1, 1)
        button.text:SetFont(button.text:GetFont(), 12, "OUTLINE")
        button.text:SetJustifyH("LEFT")
        button.text:SetJustifyV("CENTER")
        button.text:SetWordWrap(false)
        button.text:SetPoint("RIGHT", icon)
        button.text:SetPoint("BOTTOM", icon, 0, 3)

        button.ilvl = button:CreateFontString(nil, "ARTWORK", "GameFontNormal")
        button.ilvl:SetFont(button.ilvl:GetFont(), 12, "OUTLINE")
        button.ilvl:SetJustifyH("LEFT")
        button.ilvl:SetJustifyV("CENTER")
        button.ilvl:SetWordWrap(false)
        button.ilvl:SetPoint("LEFT", icon, 1)
        button.ilvl:SetPoint("BOTTOM", icon, 0, 2)

        button.leftstring = button:CreateFontString(nil, "OVERLAY", "GameFontNormal")
        button.leftstring:SetJustifyH("LEFT")
        button.leftstring:SetJustifyV("CENTER")
        button.leftstring:SetWordWrap(false)

        button.rightstring = button:CreateFontString(nil, "OVERLAY", "GameFontNormal")
        button.rightstring:SetJustifyH("LEFT")
        button.rightstring:SetJustifyV("CENTER")
        button.rightstring:SetPoint("RIGHT", -10, 0)
        button.rightstring:SetWordWrap(false)
        button.rightstring:SetTextColor(1, 1, 1)

        button.rightstring.bg = button:CreateTexture(nil, "ARTWORK")
        button.rightstring.bg:SetPoint("RIGHT", -2, 0)
        button.rightstring.bg:SetHeight(button:GetHeight() - 4)
        button.rightstring.bg:SetWidth(45)
        button.rightstring.bg:SetTexture("Interface\\Buttons\\WHITE8x8")
        button.rightstring.bg:SetVertexColor(0.05, 0.05, 0.05, 1)

        return button
    end

    BWI.Tab = {}
    function BWI.Tab.Create(parent)

        local button = CreateFrame("Button", nil, parent)
        button:SetHeight(32)
        button:SetWidth(33)
        button:SetFrameStrata("HIGH")

        button:SetBackdrop({bgFile = "Interface\\Buttons\\WHITE8x8", edgeFile = "Interface\\Buttons\\WHITE8x8", edgeSize = 1})
        button:SetBackdropColor(0, 0, 0, 0.2)
        button:SetBackdropBorderColor(0, 0, 0, 1)

        local normal = button:CreateTexture(nil, "BACKGROUND")

        normal:SetHeight(30)
        normal:SetWidth(30)
        normal:SetPoint("CENTER")
        normal:SetTexture("Interface\\AddOns\\BanksWatch\\media\\gloss")
        normal:SetTexCoord(0.1, 0.9, 0.1, 0.9)
        button:SetNormalTexture(normal)

        return button
    end

    function BWI.Button:Enter(...)

        if self.link then
            GameTooltip:SetOwner(self, "ANCHOR_NONE")
            GameTooltip:SetHyperlink(self.link)
            GameTooltip:ClearAllPoints()
            GameTooltip:SetPoint("TOPRIGHT", self, "TOPLEFT", -10, -20)
            if self.quest then
                GameTooltip:SetBackdropBorderColor(1, 1, 0)
            elseif self.quality then
                local r, g, b = GetItemQualityColor(self.quality)
                GameTooltip:SetBackdropBorderColor(r, g, b)
            end
            GameTooltip:Show()
        end
    end

    function BWI.Backdrop(frame)

        frame:SetBackdrop({  bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
                                    edgeFile = "Interface\\ChatFrame\\ChatFrameBackground",
                                    tile = false, tileSize = 32, edgeSize = 1,
                                    insets = { left = 0, right = 0, top = 0, bottom = 0 }
        })
        frame:SetBackdropBorderColor(0, 0, 0, 1)
        frame:SetBackdropColor(0, 0, 0, 0.4)
        frame:EnableMouse(true)
        frame:SetClampedToScreen(true)
    end

    BWI.GradientButton = {}
    function BWI.GradientButton.Create(button, orientation)

        button:SetNormalTexture("")
        button:SetHighlightTexture("")
        button:SetPushedTexture("")
        button:SetDisabledTexture("")

        BW_RemoveEdge(button)

        button:SetBackdrop({    bgFile = AS_backdrop,
                                edgeFile = AS_backdrop,
                                edgeSize = 1,
                                insets = { left = 0, right = 0, top = 0, bottom = 0 }
        })
        button:SetBackdropColor(0, 0, 0, 0)
        button:SetBackdropBorderColor(0, 0, 0, 1)

        local gradient = button:CreateTexture(nil, "BACKGROUND")

        gradient:SetColorTexture(1, 1, 1)
        gradient:SetAllPoints()
        gradient:SetGradient(orientation, 0, 0, 0, 0.15, 0.15, 0.15)
    end

    function BW_RemoveEdge(frame)

        local name = frame:GetName()
        local left = frame.Left or name and _G[name.."Left"] or nil
        local middle = frame.Middle or name and (_G[name.."Middle"] or _G[name.."Mid"]) or nil
        local right = frame.Right or name and _G[name.."Right"] or nil

        if left then left:Hide() end
        if middle then middle:Hide() end
        if right then right:Hide() end
    end

    function BW_AnchorBagsTopLeft()
        local x, lasttab
        -- DEFAULT LAYOUT
        BW.mainframe.layout = 1
        BWI.Main.Frame.ButtonOrder("LEFT")
        -- bank
        BW.mainframe.bank:ClearAllPoints()
        BW.mainframe.bank:SetPoint("TOPLEFT", BW.mainframe, "BOTTOMLEFT")
        BWI.Main.Frame.ButtonHighlight(BW.mainframe.bankbutton, "BOTTOM")

        -- reagent
        BW.mainframe.reagent:ClearAllPoints()
        BW.mainframe.reagent:SetPoint("TOPLEFT", BW.mainframe, "BOTTOMLEFT")
        BWI.Main.Frame.ButtonHighlight(BW.mainframe.reagentbutton, "BOTTOM")

        -- void
        BW.mainframe.void:ClearAllPoints()
        BW.mainframe.void:SetPoint("TOPLEFT", BW.mainframe, "BOTTOMLEFT")
        BWI.Main.Frame.ButtonHighlight(BW.mainframe.voidbutton, "BOTTOM")
        for x = 1, BW_VOID_PAGES do
            BW.mainframe.void[x].tab:ClearAllPoints()
            if x == 1 then
                BW.mainframe.void[x].tab:SetPoint("TOPLEFT", BW.mainframe.void, "TOPRIGHT", 0, -1)
            else
                BW.mainframe.void[x].tab:SetPoint("TOPLEFT", lasttab, "BOTTOMLEFT", 0, -4)
            end
            lasttab = BW.mainframe.void[x].tab
        end

        -- guild
        BW.mainframe.guild:ClearAllPoints()
        BW.mainframe.guild:SetPoint("TOPLEFT", BW.mainframe, "BOTTOMLEFT")
        BWI.Main.Frame.ButtonHighlight(BW.mainframe.guildbutton, "BOTTOM")
        for x = 1, MAX_GUILDBANK_TABS do
            BW.mainframe.guild[x].tab:ClearAllPoints()
            if x == 1 then
                BW.mainframe.guild[x].tab:SetPoint("TOPLEFT", BW.mainframe.guild, "TOPRIGHT", 0, -1)
                BW.mainframe.guild[x].tab:SetBackdropBorderColor(1, 1, 1, 1)
            else
                BW.mainframe.guild[x].tab:SetPoint("TOPLEFT", lasttab, "BOTTOMLEFT", 0, -4)
            end
            lasttab = BW.mainframe.guild[x].tab
        end

        -- search
        BW.mainframe.search:ClearAllPoints()
        BW.mainframe.search:SetPoint("TOPLEFT", BW.mainframe.optionbutton, "BOTTOMLEFT")
        BW.mainframe.search:SetPoint("TOPRIGHT", BW.mainframe.countbutton, "BOTTOMRIGHT")
        BW.mainframe.search.enddot:SetPoint("TOP", BW.mainframe.search, "BOTTOM", 0, -5)
        BWI.Main.Frame.ButtonHighlight(BW.mainframe.countbutton, "BOTTOM")
    end

    function BW_AnchorBagsBottomLeft()

        BW.mainframe.layout = 2
        BWI.Main.Frame.ButtonOrder("LEFT")
        -- bank
        BW.mainframe.bank:ClearAllPoints()
        BW.mainframe.bank:SetPoint("BOTTOMLEFT", BW.mainframe, "TOPLEFT")
        BWI.Main.Frame.ButtonHighlight(BW.mainframe.bankbutton, "TOP")

        -- reagent
        BW.mainframe.reagent:ClearAllPoints()
        BW.mainframe.reagent:SetPoint("BOTTOMLEFT", BW.mainframe, "TOPLEFT")
        BWI.Main.Frame.ButtonHighlight(BW.mainframe.reagentbutton, "TOP")

        -- void
        BW.mainframe.void:ClearAllPoints()
        BW.mainframe.void:SetPoint("BOTTOMLEFT", BW.mainframe, "TOPLEFT")
        BWI.Main.Frame.ButtonHighlight(BW.mainframe.voidbutton, "TOP")
        for x = 1, BW_VOID_PAGES do
            BW.mainframe.void[x].tab:ClearAllPoints()
            if x == 1 then
                BW.mainframe.void[x].tab:SetPoint("TOPLEFT", BW.mainframe.void, "TOPRIGHT", 0, -1)
            else
                BW.mainframe.void[x].tab:SetPoint("TOPLEFT", lasttab, "BOTTOMLEFT", 0, -4)
            end
            lasttab = BW.mainframe.void[x].tab
        end

        -- guild
        BW.mainframe.guild:ClearAllPoints()
        BW.mainframe.guild:SetPoint("BOTTOMLEFT", BW.mainframe, "TOPLEFT")
        BWI.Main.Frame.ButtonHighlight(BW.mainframe.guildbutton, "TOP")
        for x = 1, MAX_GUILDBANK_TABS do
            BW.mainframe.guild[x].tab:ClearAllPoints()
            if x == 1 then
                BW.mainframe.guild[x].tab:SetPoint("TOPLEFT", BW.mainframe.guild, "TOPRIGHT", 0, -1)
                BW.mainframe.guild[x].tab:SetBackdropBorderColor(1, 1, 1, 1)
            else
                BW.mainframe.guild[x].tab:SetPoint("TOPLEFT", lasttab, "BOTTOMLEFT", 0, -4)
            end
            lasttab = BW.mainframe.guild[x].tab
        end

        -- search
        BW.mainframe.search:ClearAllPoints()
        BW.mainframe.search:SetPoint("BOTTOMLEFT", BW.mainframe.optionbutton, "TOPLEFT")
        BW.mainframe.search:SetPoint("BOTTOMRIGHT", BW.mainframe.countbutton, "TOPRIGHT")
        BW.mainframe.search.enddot:SetPoint("TOP", BW.mainframe.search, "TOP", 0, 5)
        BWI.Main.Frame.ButtonHighlight(BW.mainframe.countbutton, "TOP")
    end

    function BW_AnchorBagsTopRight()

        BW.mainframe.layout = 3
        BWI.Main.Frame.ButtonOrder("RIGHT")
        -- bank
        BW.mainframe.bank:ClearAllPoints()
        BW.mainframe.bank:SetPoint("TOPRIGHT", BW.mainframe, "BOTTOMRIGHT")
        BWI.Main.Frame.ButtonHighlight(BW.mainframe.bankbutton, "BOTTOM")

        -- reagent
        BW.mainframe.reagent:ClearAllPoints()
        BW.mainframe.reagent:SetPoint("TOPRIGHT", BW.mainframe, "BOTTOMRIGHT")
        BWI.Main.Frame.ButtonHighlight(BW.mainframe.reagentbutton, "BOTTOM")

        -- void
        BW.mainframe.void:ClearAllPoints()
        BW.mainframe.void:SetPoint("TOPRIGHT", BW.mainframe, "BOTTOMRIGHT")
        BWI.Main.Frame.ButtonHighlight(BW.mainframe.voidbutton, "BOTTOM")
        for x = 1, BW_VOID_PAGES do
            BW.mainframe.void[x].tab:ClearAllPoints()
            if x == 1 then
                BW.mainframe.void[x].tab:SetPoint("TOPRIGHT", BW.mainframe.void, "TOPLEFT", 0, -1)
            else
                BW.mainframe.void[x].tab:SetPoint("TOPRIGHT", lasttab, "BOTTOMRIGHT", 0, -4)
            end
            lasttab = BW.mainframe.void[x].tab
        end

        -- guild
        BW.mainframe.guild:ClearAllPoints()
        BW.mainframe.guild:SetPoint("TOPRIGHT", BW.mainframe, "BOTTOMRIGHT")
        BWI.Main.Frame.ButtonHighlight(BW.mainframe.guildbutton, "BOTTOM")
        for x = 1, MAX_GUILDBANK_TABS do
            BW.mainframe.guild[x].tab:ClearAllPoints()
            if x == 1 then
                BW.mainframe.guild[x].tab:SetPoint("TOPRIGHT", BW.mainframe.guild, "TOPLEFT", 0, -1)
                BW.mainframe.guild[x].tab:SetBackdropBorderColor(1, 1, 1, 1)
            else
                BW.mainframe.guild[x].tab:SetPoint("TOPRIGHT", lasttab, "BOTTOMRIGHT", 0, -4)
            end
            lasttab = BW.mainframe.guild[x].tab
        end

        -- search
        BW.mainframe.search:ClearAllPoints()
        BW.mainframe.search:SetPoint("TOPRIGHT", BW.mainframe.optionbutton, "BOTTOMRIGHT")
        BW.mainframe.search:SetPoint("TOPLEFT", BW.mainframe.countbutton, "BOTTOMLEFT")
        BW.mainframe.search.enddot:SetPoint("TOP", BW.mainframe.search, "BOTTOM", 0, -5)
        BWI.Main.Frame.ButtonHighlight(BW.mainframe.countbutton, "BOTTOM")
    end

    function BW_AnchorBagsBottomRight()

        BW.mainframe.layout = 4
        BWI.Main.Frame.ButtonOrder("RIGHT")
        -- bank
        BW.mainframe.bank:ClearAllPoints()
        BW.mainframe.bank:SetPoint("BOTTOMRIGHT", BW.mainframe, "TOPRIGHT")
        BWI.Main.Frame.ButtonHighlight(BW.mainframe.bankbutton, "TOP")

        -- reagent
        BW.mainframe.reagent:ClearAllPoints()
        BW.mainframe.reagent:SetPoint("BOTTOMRIGHT", BW.mainframe, "TOPRIGHT")
        BWI.Main.Frame.ButtonHighlight(BW.mainframe.reagentbutton, "TOP")

        -- void
        BW.mainframe.void:ClearAllPoints()
        BW.mainframe.void:SetPoint("BOTTOMRIGHT", BW.mainframe, "TOPRIGHT")
        BWI.Main.Frame.ButtonHighlight(BW.mainframe.voidbutton, "TOP")
        for x = 1, BW_VOID_PAGES do
            BW.mainframe.void[x].tab:ClearAllPoints()
            if x == 1 then
                BW.mainframe.void[x].tab:SetPoint("TOPRIGHT", BW.mainframe.void, "TOPLEFT", 0, -1)
            else
                BW.mainframe.void[x].tab:SetPoint("TOPRIGHT", lasttab, "BOTTOMRIGHT", 0, -4)
            end
            lasttab = BW.mainframe.void[x].tab
        end

        -- guild
        BW.mainframe.guild:ClearAllPoints()
        BW.mainframe.guild:SetPoint("BOTTOMRIGHT", BW.mainframe, "TOPRIGHT")
        BWI.Main.Frame.ButtonHighlight(BW.mainframe.guildbutton, "TOP")
        for x = 1, MAX_GUILDBANK_TABS do
            BW.mainframe.guild[x].tab:ClearAllPoints()
            if x == 1 then
                BW.mainframe.guild[x].tab:SetPoint("TOPRIGHT", BW.mainframe.guild, "TOPLEFT", 0, -1)
                BW.mainframe.guild[x].tab:SetBackdropBorderColor(1, 1, 1, 1)
            else
                BW.mainframe.guild[x].tab:SetPoint("TOPRIGHT", lasttab, "BOTTOMRIGHT", 0, -4)
            end
            lasttab = BW.mainframe.guild[x].tab
        end

        -- search
        BW.mainframe.search:ClearAllPoints()
        BW.mainframe.search:SetPoint("BOTTOMRIGHT", BW.mainframe.optionbutton, "TOPRIGHT")
        BW.mainframe.search:SetPoint("BOTTOMLEFT", BW.mainframe.countbutton, "TOPLEFT")
        BW.mainframe.search.enddot:SetPoint("TOP", BW.mainframe.search, "TOP", 0, 5)
        BWI.Main.Frame.ButtonHighlight(BW.mainframe.countbutton, "TOP")
    end

    function BW_ScrollFrame(parent, name, onUpdate, buttons)

        local scrollframe = BWI.Scroll.Create(parent, name, onUpdate)
        local i, currentrow, previousrow

        parent.buttons = {}

        for i = 1, BWrowsthatcanfit() do
            parent.buttons[i] = buttons(i)
            currentrow = parent.buttons[i]
            if i == 1 then
                currentrow:SetPoint("TOP")
            else
                currentrow:SetPoint("TOP", previousrow, "BOTTOM", 0, -1)
            end
            currentrow:Show()
            previousrow = currentrow
        end

        return scrollframe
    end

    BWI.Scroll = {}
    function BWI.Scroll.Create(parent, name, onUpdate)

        --[[note the anchors: the area of the scrollframe is the scrollable area
            that intercepts mousewheel to scroll. it does not include the scrollbar,
            which is anchored off the right ]]

        local scroll = CreateFrame("ScrollFrame", name, parent, "FauxScrollFrameTemplate")
        scroll:SetPoint("TOPLEFT", BW.mainframe.search, "TOPLEFT")
        scroll:SetPoint("BOTTOMRIGHT", BW.mainframe.search, "BOTTOMRIGHT")
        scroll.onUpdate = onUpdate

        scroll:SetScript("OnShow", onUpdate)
        scroll:SetScript("OnVerticalScroll", BWI.Scroll.VerticalScroll)
        scroll.ScrollBar:Hide()

        return scroll
    end

    function BWI.Scroll:VerticalScroll(offset)
        FauxScrollFrame_OnVerticalScroll(self, offset, 32, self.onUpdate)
    end
